@@ -6,8 +6,10 @@ import collections
 from . import arc
 from . import entities
 
+from .. import util
+
 from ..nsphere import fit_nsphere
-from ..util import unitize, diagonal_dot
+
 from ..constants import log
 from ..constants import tol_path as tol
 
@@ -27,15 +29,18 @@ def fit_circle_check(points,
 
     Parameters
     ---------
-    points:  (n, d) set of points which represent a path
-    prior:   (center, radius) tuple for best guess, or None if unknown
-    scale:   float, what is the overall scale of the set of points
-    verbose: boolean, if True output log.debug messages for the reasons
-             for fit rejection. Potentially generates hundreds of thousands of
-             messages so only suggested in manual debugging.
+    points :  (n, d)
+      List of points which represent a path
+    prior :  (center, radius) tuple
+      Best guess or None if unknown
+    scale : float
+      What is the overall scale of the set of points
+    verbose : bool
+     Output log.debug messages for the reasons
+     for fit rejection only suggested for manual debugging
 
     Returns
-    ---------
+    -----------
     if fit is acceptable:
         (center, radius) tuple
     else:
@@ -62,12 +67,11 @@ def fit_circle_check(points,
         return None
 
     vectors = np.diff(points, axis=0)
-    segment = np.linalg.norm(vectors, axis=1)
+    segment = util.row_norm(vectors)
 
     # approximate angle in radians, segments are linear length
     # not arc length but this is close and avoids a cosine
     angle = segment / R
-
     if (angle > tol.seg_angle).any():
         if verbose:
             log.debug('circle fit error: angle %s', str(angle))
@@ -88,10 +92,11 @@ def fit_circle_check(points,
     # check to make sure the line segments on the ends are actually
     # tangent with the candidate circle fit
     mid_pt = points[[0, -2]] + (vectors[[0, -1]] * .5)
-    radial = unitize(mid_pt - C)
-    ends = unitize(vectors[[0, -1]])
-    tangent = np.abs(np.arccos(diagonal_dot(radial, ends)))
+    radial = util.unitize(mid_pt - C)
+    ends = util.unitize(vectors[[0, -1]])
+    tangent = np.abs(np.arccos(util.diagonal_dot(radial, ends)))
     tangent = np.abs(tangent - np.pi / 2).max()
+
     if tangent > tol.tangent:
         if verbose:
             log.debug('circle fit error: tangent %f',
@@ -111,9 +116,12 @@ def is_circle(points, scale, verbose=False):
 
     Parameters
     -------------
-    points: (n,2) float, points in space
-    scale:  float, scale of overall drawing
-    verbose: bool, print all fit messages or not
+    points : (n,2 ) float
+      Points in space
+    scale : float
+      Scale of overall drawing
+    verbose : bool
+      Print all fit messages or not
 
     Returns
     -------------
@@ -154,13 +162,16 @@ def merge_colinear(points, scale):
 
     Parameters
     ----------
-    points: (n, d) set of points (where d is dimension)
-    scale:  float, scale of drawing
+    points : (n, dimension) float
+      Points in space
+    scale : float
+      Scale of drawing for precision
 
     Returns
     ----------
-    merged: (j, d) set of points with colinear and duplicate
-             points merged, where (j < n)
+    merged : (j, d) float
+      Points with colinear and duplicate
+      points merged, where (j < n)
     """
     points = np.asanyarray(points, dtype=np.float64)
     scale = float(scale)
@@ -175,7 +186,7 @@ def merge_colinear(points, scale):
     # the vector from one point to the next
     direction = points[1:] - points[:-1]
     # the length of the direction vector
-    direction_norm = np.linalg.norm(direction, axis=1)
+    direction_norm = util.row_norm(direction)
     # make sure points don't have zero length
     direction_ok = direction_norm > tol.merge
 
@@ -189,14 +200,14 @@ def merge_colinear(points, scale):
     # and direction vectors A-B, B-C, etc
     # these will be perpendicular to the vectors A-C, B-D, etc
     perp = (points[2:] - points[:-2]).T[::-1].T
-    perp_norm = np.linalg.norm(perp, axis=1)
+    perp_norm = util.row_norm(perp)
     perp_nonzero = perp_norm > tol.merge
     perp[perp_nonzero] /= perp_norm[perp_nonzero].reshape((-1, 1))
 
     # find the projection of each direction vector
     # onto the perpendicular vector
-    projection = np.abs(diagonal_dot(perp,
-                                     direction[:-1]))
+    projection = np.abs(util.diagonal_dot(perp,
+                                          direction[:-1]))
 
     projection_ratio = np.max((projection / direction_norm[1:],
                                projection / direction_norm[:-1]), axis=0)
@@ -215,14 +226,19 @@ def resample_spline(points, smooth=.001, count=None, degree=3):
 
     Parameters
     -----------
-    points: (n, dimension) float, points in space
-    smooth: float, smoothing amount
-    count:  number of samples in output
-    degree: int, degree of spline polynomial
+    points : (n, dimension) float
+      Points in space
+    smooth : float
+      Smoothing distance
+    count :  int or None
+      Number of samples desired in output
+    degree : int
+      Degree of spline polynomial
 
     Returns
     ---------
-    resampled: (count, dimension) float, points in space
+    resampled : (count, dimension) float
+      Points in space
     """
     from scipy.interpolate import splprep, splev
     if count is None:
@@ -248,14 +264,19 @@ def points_to_spline_entity(points, smooth=None, count=None):
 
     Parameters
     -----------
-    points: (n, dimension) float, points in space
-    smooth: float, smoothing amount
-    count:  int, number of samples in result
+    points : (n, dimension) float
+      Points in space
+    smooth : float
+      Smoothing distance
+    count :  int or None
+      Number of samples desired in result
 
     Returns
     ---------
-    entity: entities.BSpline object with points indexed at zero
-    control: (m, dimension) float, new vertices for entity
+    entity : entities.BSpline
+      Entity object with points indexed at zero
+    control : (m, dimension) float
+      New vertices for entity
     """
 
     from scipy.interpolate import splprep
@@ -289,11 +310,13 @@ def simplify_basic(drawing, process=False, **kwargs):
 
     Parameters
     -----------
-    drawing: Path2D object, will not be modified.
+    drawing : Path2D
+      Source geometry, will not be modified
 
     Returns
     -----------
-    simplified: Path2D with circles.
+    simplified : Path2D
+      Original path but with some closed line-loops converted to circles
     """
 
     if any(i.__class__.__name__ != 'Line'
