@@ -1,5 +1,5 @@
 """
-visual.py
+color.py
 -------------
 
 Hold and deal with visual information about meshes.
@@ -75,7 +75,7 @@ class ColorVisuals(object):
         if vertex_colors is not None:
             self.vertex_colors = vertex_colors
 
-    @property
+    @caching.cache_decorator
     def transparency(self):
         """
         Does the current object contain any transparency.
@@ -274,12 +274,14 @@ class ColorVisuals(object):
         colors: (count, 4) uint8, RGBA colors
         """
 
+        count = None
         try:
-            counts = {'face': len(self.mesh.faces),
-                      'vertex': len(self.mesh.vertices)}
-            count = counts[name]
-        except AttributeError:
-            count = None
+            if name == 'face':
+                count = len(self.mesh.faces)
+            elif name == 'vertex':
+                count = len(self.mesh.vertices)
+        except BaseException:
+            pass
 
         # the face or vertex colors
         key_colors = str(name) + '_colors'
@@ -489,34 +491,37 @@ def to_rgba(colors, dtype=np.uint8):
 
     Parameters
     ----------
-    colors: (n,[3|4]) list of RGB or RGBA colors
+    colors : (n, 3) or (n, 4) array
+      RGB or RGBA colors
 
     Returns
     ----------
-    colors: (n,4) list of RGBA colors
-            (4,)  single RGBA color
+    colors : (n, 4) list of RGBA colors
+             (4,)  single RGBA color
     """
-    if not util.is_sequence(colors):
-        return
+    if colors is None or not util.is_sequence(colors):
+        return DEFAULT_COLOR
+
     # colors as numpy array
     colors = np.asanyarray(colors)
 
-    # integer value for opaque given our datatype
-    opaque = (2**(np.dtype(dtype).itemsize * 8)) - 1
+    # integer value for opaque alpha given our datatype
+    opaque = np.iinfo(dtype).max
 
-    if (colors.dtype.kind == 'f' and
-            colors.max() < (1.0 + 1e-8)):
-        colors = (colors * opaque).astype(dtype)
-    elif (colors.dtype.kind in 'iu' and
-          colors.max() <= opaque):
+    if (colors.dtype.kind == 'f' and colors.max() < (1.0 + 1e-8)):
+        colors = (colors * opaque).round().astype(dtype)
+    elif (colors.max() <= opaque):
         colors = colors.astype(dtype)
     else:
         raise ValueError('colors non- convertible!')
 
     if util.is_shape(colors, (-1, 3)):
-        colors = np.column_stack((colors,
-                                  opaque * np.ones(len(colors)))).astype(dtype)
+        # add an opaque alpha for RGB colors
+        colors = np.column_stack((
+            colors,
+            opaque * np.ones(len(colors)))).astype(dtype)
     elif util.is_shape(colors, (3,)):
+        # if passed a single RGB color add an alpha
         colors = np.append(colors, opaque)
 
     if not (util.is_shape(colors, (4,)) or
@@ -526,6 +531,33 @@ def to_rgba(colors, dtype=np.uint8):
     return colors
 
 
+def to_float(colors):
+    """
+    Convert integer colors to 0.0 - 1.0 floating point colors
+
+    Parameters
+    -------------
+    colors : (n, d) int
+      Integer colors
+
+    Returns
+    -------------
+    as_float : (n, d) float
+      Float colors 0.0 - 1.0
+    """
+
+    # colors as numpy array
+    colors = np.asanyarray(colors)
+    if colors.dtype.kind == 'f':
+        return colors
+    elif colors.dtype.kind in 'iu':
+        # integer value for opaque alpha given our datatype
+        opaque = np.iinfo(colors.dtype).max
+        return colors.astype(np.float64) / opaque
+    else:
+        raise ValueError('only works on int or float colors!')
+
+
 def hex_to_rgba(color):
     """
     Turn a string hex color to a (4,) RGBA color.
@@ -588,13 +620,16 @@ def vertex_to_face_color(vertex_colors, faces):
     return face_colors.astype(np.uint8)
 
 
-def face_to_vertex_color(mesh, face_colors, dtype=np.uint8):
+def face_to_vertex_color(
+        mesh,
+        face_colors,
+        dtype=np.uint8):
     """
-    Convert a list of face colors into a list of vertex colors.
+    Convert face colors into vertex colors.
 
     Parameters
     -----------
-    mesh:        Trimesh object
+    mesh : trimesh.Trimesh object
     face_colors: (n, (3,4)) int, face colors
     dtype:       data type of output
 
@@ -603,28 +638,33 @@ def face_to_vertex_color(mesh, face_colors, dtype=np.uint8):
     vertex_colors: (m,4) dtype, colors for each vertex
     """
     rgba = to_rgba(face_colors)
-    vertex_colors = mesh.faces_sparse.dot(
-        rgba.astype(np.float64))
-    vertex_colors /= mesh.faces_sparse.sum(axis=1)
-    vertex_colors = vertex_colors.astype(dtype)
+    vertex = mesh.faces_sparse.dot(rgba.astype(np.float64))
+    vertex = (vertex / mesh.vertex_degree.reshape(
+        (-1, 1))).astype(dtype)
 
-    return vertex_colors
+    assert vertex.shape == (len(mesh.vertices), 4)
+
+    return vertex
 
 
 def colors_to_materials(colors, count=None):
     """
-    Convert a list of colors into a list of unique materials and material
-    indexes.
+    Convert a list of colors into a list of unique materials
+    and material indexes.
 
     Parameters
     -----------
-    colors: (n,(3,4)) colors
-    count:  int, number of entities to apply color to
+    colors : (n, 3) or (n, 4) float
+      RGB or RGBA colors
+    count : int
+      Number of entities to apply color to
 
     Returns
     -----------
-    diffuse: (m,4) int, colors
-    index:   (count,) int, index of each color
+    diffuse : (m, 4) int
+      Colors
+    index : (count,) int
+      Index of each color
     """
 
     # convert RGB to RGBA
@@ -731,4 +771,45 @@ def interpolate(values, color_map=None, dtype=np.uint8):
     return rgba
 
 
+def uv_to_color(uv, image):
+    """
+    Get the color in a texture image.
+
+    Parameters
+    -------------
+    uv : (n, 2) float
+      UV coordinates on texture image
+    image : PIL.Image
+      Texture image
+
+    Returns
+    ----------
+    colors : (n, 4) float
+      RGBA color at each of the UV coordinates
+    """
+    if image is None or uv is None:
+        return None
+
+    # UV coordinates should be (n, 2) float
+    uv = np.asanyarray(uv, dtype=np.float64)
+
+    # get texture image pixel positions of UV coordinates
+    x = (uv[:, 0] * (image.width - 1))
+    y = ((1 - uv[:, 1]) * (image.height - 1))
+
+    # convert to int and wrap to image
+    # size in the manner of GL_REPEAT
+    x = x.round().astype(np.int64) % image.width
+    y = y.round().astype(np.int64) % image.height
+
+    # access colors from pixel locations
+    # make sure image is RGBA before getting values
+    colors = np.asanyarray(image.convert('RGBA'))[y, x]
+
+    # conversion to RGBA should have corrected shape
+    assert colors.ndim == 2 and colors.shape[1] == 4
+
+    return colors
+
+
 DEFAULT_COLOR = np.array([102, 102, 102, 255], dtype=np.uint8)
